import { useState, useEffect } from "react";
import type { RegistryItem } from "../lib/types";

const RegistryList = () => {
	const [registryItems, setRegistryItems] = useState<RegistryItem[]>([]);
	const [claimedItems, setClaimedItems] = useState<Set<string>>(new Set());
	const [loading, setLoading] = useState(true);
	const [error, setError] = useState<string | null>(null);
	const [claimantName, setClaimantName] = useState("");
	const [showClaimants, setShowClaimants] = useState<Set<string>>(new Set());

	useEffect(() => {
		fetchRegistryItems();
	}, []);

	const fetchRegistryItems = async () => {
		setLoading(true);
		setError(null);
		try {
			const response = await fetch("/api/registry");
			if (!response.ok) {
				throw new Error(`HTTP error! status: ${response.status}`);
			}
			const data = await response.json();
			setRegistryItems(data);
		} catch (e: any) {
			setError(e.message);
			console.error("Failed to fetch registry items:", e);
		} finally {
			setLoading(false);
		}
	};

	const handleCheckboxChange = (itemId: string) => {
		const newClaimedItems = new Set(claimedItems);
		if (newClaimedItems.has(itemId)) {
			newClaimedItems.delete(itemId);
		} else {
			newClaimedItems.add(itemId);
		}
		setClaimedItems(newClaimedItems);
	};

	const toggleClaimantVisibility = (itemId: string) => {
		const newShowClaimants = new Set(showClaimants);
		if (newShowClaimants.has(itemId)) {
			newShowClaimants.delete(itemId);
		} else {
			newShowClaimants.add(itemId);
		}
		setShowClaimants(newShowClaimants);
	};

	const handleSubmit = async () => {
		if (!claimantName.trim()) {
			setError("Please enter your name before claiming items");
			return;
		}

		const updates = registryItems.filter((item) => claimedItems.has(item.id));

		const updatedRegistryItems = registryItems.map((item) => {
			if (claimedItems.has(item.id)) {
				return { ...item, taken: true, claimedBy: claimantName };
			}
			return item;
		});
		setRegistryItems(updatedRegistryItems);
		setClaimedItems(new Set());
		setClaimantName("");

		try {
			for (const item of updates) {
				const response = await fetch(`/api/registry/${item.id}`, {
					method: "PUT",
					headers: {
						"Content-Type": "application/json",
					},
					body: JSON.stringify({ taken: true, claimedBy: claimantName }),
				});

				if (!response.ok) {
					throw new Error(
						`Failed to update item ${item.id}: ${response.statusText}`
					);
				}
			}
			alert("Thank you for claiming items!");
		} catch (error: any) {
			console.error("Error updating items:", error);
			setError("Failed to update items. Please try again.");
			setRegistryItems(registryItems);
		}
	};

	if (loading) {
		return <div className="text-center">Loading registry items...</div>;
	}

	if (error) {
		return <div className="text-red-500 text-center">Error: {error}</div>;
	}

	return (
		<div className="container mx-auto p-4">
			<h1 className="text-2xl font-bold mb-4">Baby Registry</h1>
			<div className="overflow-x-auto">
				<table className="table w-full">
					<thead>
						<tr>
							<th>Item</th>
							<th>Link</th>
							<th>Status</th>
							<th>Claim</th>
						</tr>
					</thead>
					<tbody>
						{registryItems.map((item) => (
							<tr key={item.id}>
								<td>
									<div className="flex flex-col gap-1">
										<div className="flex items-center gap-2">
											{item.name}
											{item.taken && (
												<div className="flex items-center gap-2">
													<span className="badge badge-success">Taken</span>
													<button
														onClick={() => toggleClaimantVisibility(item.id)}
														className="btn btn-xs btn-ghost"
													>
														{showClaimants.has(item.id) ? "Hide" : "Show"}
													</button>
												</div>
											)}
										</div>
										{item.taken && 
											item.claimedBy && 
											showClaimants.has(item.id) && (
												<div className="text-sm text-gray-500 italic">
													Claimed by: {item.claimedBy}
												</div>
											)}
									</div>
								</td>
								<td>
									{item.link && (
										<a
											href={item.link}
											target="_blank"
											rel="noopener noreferrer"
											className="link link-primary"
										>
											View
										</a>
									)}
								</td>
								<td>{item.taken ? "Claimed" : "Available"}</td>
								<td>
									{!item.taken ? (
										<input
											type="checkbox"
											className="checkbox"
											checked={claimedItems.has(item.id)}
											onChange={() => handleCheckboxChange(item.id)}
										/>
									) : <input
										type="checkbox"
										className="checkbox"
										disabled
										checked={true}
									/>}
								</td>
							</tr>
						))}
					</tbody>
				</table>
			</div>
			{claimedItems.size > 0 && (
				<div className="mt-4 flex flex-col gap-4">
					<div className="form-control">
						<label className="label">
							<span className="label-text">Your Name</span>
						</label>
						<input
							type="text"
							className="input input-bordered"
							value={claimantName}
							onChange={(e) => setClaimantName(e.target.value)}
							placeholder="Enter your name"
							required
						/>
					</div>
					<button
						className="btn btn-primary"
						onClick={handleSubmit}
						disabled={!claimantName.trim()}
					>
						Submit Claims
					</button>
				</div>
			)}
		</div>
	);
};

export default RegistryList;
