import {
	S3Client,
	GetObjectCommand,
	PutObjectCommand,
} from "@aws-sdk/client-s3";

const s3Host = process.env.S3_HOST || import.meta.env.S3_HOST;
const endpoint = `https://${s3Host}`;

const s3Client = new S3Client({
	region: process.env.S3_REGION || import.meta.env.S3_REGION,
	endpoint,
	credentials: {
		accessKeyId: process.env.S3_ACCESS_KEY || import.meta.env.S3_ACCESS_KEY,
		secretAccessKey:
			process.env.S3_SECRET_KEY || import.meta.env.S3_SECRET_KEY,
	},
	forcePathStyle: true,
});

const BUCKET_NAME =
	process.env.S3_BUCKET_NAME || import.meta.env.S3_BUCKET_NAME;

export async function getS3Data<T>(key: string): Promise<T | undefined> {
	try {
		const getCommand = new GetObjectCommand({
			Bucket: BUCKET_NAME,
			Key: key,
		});

		const response = await s3Client.send(getCommand);
		const fileContent = await response.Body?.transformToString();

		if (fileContent) {
			return JSON.parse(fileContent) as T;
		} else {
			console.warn(`File ${key} is empty.`);
			return undefined;
		}
	} catch (error: any) {
		console.warn(`Error getting data from S3 at key ${key}:`, error.message);
		return undefined;
	}
}

export async function putS3Data<T>(
	key: string,
	data: T,
	contentType: string = "application/json"
): Promise<void> {
	try {
		const body = JSON.stringify(data, null, 2); // Pretty-print JSON
		const putCommand = new PutObjectCommand({
			Bucket: BUCKET_NAME,
			Key: key,
			Body: body,
			ContentType: contentType,
		});

		await s3Client.send(putCommand);
		console.log(`Successfully saved data to S3 at key: ${key}`);
	} catch (error: any) {
		console.error(`Error putting data to S3 at key ${key}:`, error.message);
		throw error; // Re-throw to handle in the API route
	}
}
