import type { APIRoute } from "astro";
import { getS3Data, putS3Data } from "../../../lib/s3";
import type { RegistryItem } from "../../../lib/types";

const REGISTRY_FILE_KEY = "registry.json";

// GET: Get a specific registry item by ID
export const GET: APIRoute = async ({ params }) => {
	try {
		const { id } = params;
		if (!id) {
			return new Response(
				JSON.stringify({ success: false, error: "Item ID is required" }),
				{ status: 400, headers: { "Content-Type": "application/json" } }
			);
		}

		const registry = await getS3Data<RegistryItem[]>(REGISTRY_FILE_KEY) || [];
		const item = registry.find((item) => item.id === id);

		if (!item) {
			return new Response(
				JSON.stringify({ success: false, error: "Item not found" }),
				{ status: 404, headers: { "Content-Type": "application/json" } }
			);
		}

		return new Response(JSON.stringify(item), {
			status: 200,
			headers: { "Content-Type": "application/json" },
		});
	} catch (error: any) {
		console.error("Error getting registry item:", error.message);
		return new Response(
			JSON.stringify({ success: false, error: "Failed to get item" }),
			{ status: 500, headers: { "Content-Type": "application/json" } }
		);
	}
};

// PUT: Update an existing registry item
export const PUT: APIRoute = async ({ request, params }) => {
	try {
		const { id } = params;
		if (!id) {
			return new Response(
				JSON.stringify({ success: false, error: "Item ID is required" }),
				{ status: 400, headers: { "Content-Type": "application/json" } }
			);
		}

		const body = await request.json();
		const { taken, claimedBy } = body;

		const registry = await getS3Data<RegistryItem[]>(REGISTRY_FILE_KEY) || [];
		const itemIndex = registry.findIndex((item) => item.id === id);

		if (itemIndex === -1) {
			return new Response(
				JSON.stringify({ success: false, error: "Item not found" }),
				{ status: 404, headers: { "Content-Type": "application/json" } }
			);
		}

		registry[itemIndex] = {
			...registry[itemIndex],
			taken: taken !== undefined ? taken : registry[itemIndex].taken,
			claimedBy: claimedBy !== undefined ? claimedBy : registry[itemIndex].claimedBy,
		};

		await putS3Data(REGISTRY_FILE_KEY, registry);

		return new Response(JSON.stringify({ success: true }), {
			status: 200,
			headers: { "Content-Type": "application/json" },
		});
	} catch (error: any) {
		console.error("Error updating registry item:", error.message);
		return new Response(
			JSON.stringify({ success: false, error: "Failed to update item" }),
			{ status: 500, headers: { "Content-Type": "application/json" } }
		);
	}
};


// DELETE: Delete a registry item
export const DELETE: APIRoute = async ({ params }) => {
	const headers = {
		"Content-Type": "application/json",
	};

	try {
		const { id } = params;
		if (!id) {
			return new Response(
				JSON.stringify({ success: false, error: "Item ID is required" }),
				{ status: 400, headers }
			);
		}

		// Get current registry data
		const registry = await getS3Data<RegistryItem[]>(REGISTRY_FILE_KEY) || [];
    
		// Find the item index
		const itemIndex = registry.findIndex((item) => item.id === id);

		if (itemIndex === -1) {
			return new Response(
				JSON.stringify({ success: false, error: "Item not found" }),
				{ status: 404, headers }
			);
		}

		// Remove the item
		registry.splice(itemIndex, 1);

		// Save the updated registry
		await putS3Data(REGISTRY_FILE_KEY, registry);

		return new Response(JSON.stringify({ success: true }), {
			status: 200,
			headers,
		});
	} catch (error) {
		console.error("Error deleting registry item:", error);
		return new Response(
			JSON.stringify({
				success: false,
				error: "Failed to delete item",
				details: error instanceof Error ? error.message : "Unknown error",
			}),
			{ status: 500, headers }
		);
	}
};
