import type { APIRoute } from "astro";
import { getS3Data, putS3Data } from "../../lib/s3";
import type { RSVPItem } from "../../lib/types";

const objectsToCSV = (data: RSVPItem[]): string => {
  const headers = ["name", "dietaryRestrictions", "timestamp"];
  const csvRows = [headers.join(",")];

  data.forEach((entry) => {
    const row = headers.map((header) => {
      const field = String(entry[header as keyof RSVPItem]);
      const escaped = field.replace(/"/g, '""');
      return `"${escaped}"`;
    });
    csvRows.push(row.join(","));
  });

  return csvRows.join("\n");
};

const csvToObjects = (csv: string): RSVPItem[] => {
  const lines = csv.split("\n");
  const headers = lines[0].split(",").map((h) => h.trim());

  return lines
    .slice(1)
    .filter((line) => line.trim())
    .map((line) => {
      const values = line.match(/(".*?"|[^",\s]+)(?=\s*,|\s*$)/g) || [];
      const entry: Partial<RSVPItem> = {};

      headers.forEach((header, index) => {
        let value = values[index] || "";
        value = value.replace(/^"(.*)"$/, "$1").replace(/""/g, '"');
        entry[header as keyof RSVPItem] = value;
      });

      return entry as RSVPItem;
    });
};

export const POST: APIRoute = async ({ request }) => {
  console.log("API endpoint hit - starting request processing");

  const headers = {
    "Access-Control-Allow-Origin": "*",
    "Content-Type": "application/json",
  };

  try {
    const FILE_KEY = "rsvp.csv";

    console.log("Parsing request body");
    const newRsvp = await request.json();
    console.log("Received RSVP data:", newRsvp);

    let existingRsvps: RSVPItem[] = [];

    console.log("Attempting to fetch existing RSVPs");
    const fileContent = await getS3Data<string>(FILE_KEY);
    if (fileContent) {
      existingRsvps = csvToObjects(fileContent);
      console.log("Existing RSVPs loaded:", existingRsvps.length);
    }

    existingRsvps.push({
      ...newRsvp,
      timestamp: new Date().toISOString(),
    });

    console.log("Attempting to save updated RSVP list");
    const csvContent = objectsToCSV(existingRsvps);

    await putS3Data(FILE_KEY, csvContent, "text/csv");

    return new Response(JSON.stringify({ success: true }), {
      status: 200,
      headers,
    });
  } catch (error) {
    console.error("Error handling RSVP:", error);
    return new Response(
      JSON.stringify({
        success: false,
        error: error instanceof Error ? error.message : "Unknown error",
        details: error instanceof Error ? error.stack : undefined,
      }),
      {
        status: 500,
        headers,
      }
    );
  }
};
